package javax.swing.plaf.metal;

import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import javax.swing.UIManager;
import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import javax.swing.event.MouseInputListener;
import javax.swing.plaf.UIResource;
import javax.swing.JCheckBox;
import javax.swing.JToggleButton;
import javax.swing.plaf.ComponentUI;
import java.awt.Point;
import javax.swing.JToolBar;
import javax.swing.JRootPane;
import java.awt.Component;
import javax.swing.SwingUtilities;
import javax.swing.JMenuBar;
import javax.swing.border.Border;
import java.beans.PropertyChangeListener;
import java.awt.event.ContainerListener;
import javax.swing.JComponent;
import java.lang.ref.WeakReference;
import java.util.List;
import javax.swing.plaf.basic.BasicToolBarUI;

public class MetalToolBarUI extends BasicToolBarUI
{
    private static List<WeakReference<JComponent>> components;
    protected ContainerListener contListener;
    protected PropertyChangeListener rolloverListener;
    private static Border nonRolloverBorder;
    private JMenuBar lastMenuBar;
    
    public MetalToolBarUI() {
    }
    
    static synchronized void register(JComponent referent) {
        if (referent == null) {
            throw new NullPointerException("JComponent must be non-null");
        }
        MetalToolBarUI.components.add(new WeakReference(referent));
        return;
    }
    
    static synchronized void unregister(JComponent component) {
        for (int i = MetalToolBarUI.components.size() - 1; i >= 0; --i) {
            JComponent component2 = (JComponent)((WeakReference)MetalToolBarUI.components.get(i)).get();
            if (component2 == component || component2 == null) {
                MetalToolBarUI.components.remove(i);
            }
        }
    }
    
    static synchronized Object findRegisteredComponentOfType(JComponent c, Class clazz) {
        JRootPane rootPane = SwingUtilities.getRootPane(c);
        if (rootPane != null) {
            for (int i = MetalToolBarUI.components.size() - 1; i >= 0; --i) {
                Object value = ((WeakReference)MetalToolBarUI.components.get(i)).get();
                if (value == null) {
                    MetalToolBarUI.components.remove(i);
                }
                else if (clazz.isInstance(value) && SwingUtilities.getRootPane((Component)value) == rootPane) {
                    return value;
                }
            }
        }
        return null;
    }
    
    static boolean doesMenuBarBorderToolBar(JMenuBar menuBar) {
        JToolBar source = (JToolBar)findRegisteredComponentOfType(menuBar, JToolBar.class);
        if (source != null && source.getOrientation() == 0) {
            JRootPane rootPane = SwingUtilities.getRootPane(menuBar);
            Point convertPoint = SwingUtilities.convertPoint(menuBar, new Point(0, 0), rootPane);
            int x = convertPoint.x;
            int y = convertPoint.y;
            Point point = convertPoint;
            int n = 0;
            convertPoint.y = n;
            point.x = n;
            Point convertPoint2 = SwingUtilities.convertPoint(source, convertPoint, rootPane);
            return convertPoint2.x == x && y + menuBar.getHeight() == convertPoint2.y && menuBar.getWidth() == source.getWidth();
        }
        return false;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new MetalToolBarUI();
    }
    
    @Override
    public void installUI(JComponent c) {
        super.installUI(c);
        register(c);
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        super.uninstallUI(c);
        MetalToolBarUI.nonRolloverBorder = null;
        unregister(c);
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.contListener = this.createContainerListener();
        if (this.contListener != null) {
            this.toolBar.addContainerListener(this.contListener);
        }
        this.rolloverListener = this.createRolloverListener();
        if (this.rolloverListener != null) {
            this.toolBar.addPropertyChangeListener(this.rolloverListener);
        }
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        if (this.contListener != null) {
            this.toolBar.removeContainerListener(this.contListener);
        }
        this.rolloverListener = this.createRolloverListener();
        if (this.rolloverListener != null) {
            this.toolBar.removePropertyChangeListener(this.rolloverListener);
        }
    }
    
    @Override
    protected Border createRolloverBorder() {
        return super.createRolloverBorder();
    }
    
    @Override
    protected Border createNonRolloverBorder() {
        return super.createNonRolloverBorder();
    }
    
    private Border createNonRolloverToggleBorder() {
        return this.createNonRolloverBorder();
    }
    
    @Override
    protected void setBorderToNonRollover(Component component) {
        if (component instanceof JToggleButton && !(component instanceof JCheckBox)) {
            JToggleButton toggleButton = (JToggleButton)component;
            Border border = toggleButton.getBorder();
            super.setBorderToNonRollover(component);
            if ((border instanceof UIResource)) {
                if (MetalToolBarUI.nonRolloverBorder == null) {
                    MetalToolBarUI.nonRolloverBorder = this.createNonRolloverToggleBorder();
                }
                toggleButton.setBorder(MetalToolBarUI.nonRolloverBorder);
            }
        }
        else {
            super.setBorderToNonRollover(component);
        }
    }
    
    protected ContainerListener createContainerListener() {
        return null;
    }
    
    protected PropertyChangeListener createRolloverListener() {
        return null;
    }
    
    @Override
    protected MouseInputListener createDockingListener() {
        return new MetalToolBarUI.MetalDockingListener(this.toolBar);
    }
    
    protected void setDragOffset(Point offset) {
        if (!GraphicsEnvironment.isHeadless()) {
            if (this.dragWindow == null) {
                this.dragWindow = this.createDragWindow(this.toolBar);
            }
            this.dragWindow.setOffset(offset);
        }
    }
    
    @Override
    public void update(Graphics g, JComponent c) {
        if (g == null) {
            throw new NullPointerException("graphics must be non-null");
        }
        if (c.isOpaque() && c.getBackground() instanceof UIResource && ((JToolBar)c).getOrientation() == 0 && UIManager.get("MenuBar.gradient") != null) {
            JRootPane rootPane = SwingUtilities.getRootPane(c);
            JMenuBar menuBar = (JMenuBar)findRegisteredComponentOfType(c, JMenuBar.class);
            if (menuBar != null && menuBar.isOpaque() && menuBar.getBackground() instanceof UIResource) {
                Point convertPoint = SwingUtilities.convertPoint(c, new Point(0, 0), rootPane);
                int x = convertPoint.x;
                int y = convertPoint.y;
                Point point = convertPoint;
                int n = 0;
                convertPoint.y = n;
                point.x = n;
                Point convertPoint2 = SwingUtilities.convertPoint(menuBar, convertPoint, rootPane);
                if (convertPoint2.x == x && y == convertPoint2.y + menuBar.getHeight() && menuBar.getWidth() == c.getWidth() && MetalUtils.drawGradient(c, g, "MenuBar.gradient", 0, -menuBar.getHeight(), c.getWidth(), c.getHeight() + menuBar.getHeight(), true)) {
                    this.setLastMenuBar(menuBar);
                    this.paint(g, c);
                    return;
                }
            }
            if (MetalUtils.drawGradient(c, g, "MenuBar.gradient", 0, 0, c.getWidth(), c.getHeight(), true)) {
                this.setLastMenuBar(null);
                this.paint(g, c);
                return;
            }
        }
        this.setLastMenuBar(null);
        super.update(g, c);
        return;
    }
    
    private void setLastMenuBar(JMenuBar lastMenuBar) {
        if (MetalLookAndFeel.usingOcean() && this.lastMenuBar != lastMenuBar) {
            if (this.lastMenuBar != null) {
                this.lastMenuBar.repaint();
            }
            if (lastMenuBar != null) {
                lastMenuBar.repaint();
            }
            this.lastMenuBar = lastMenuBar;
        }
    }
    
    static {
        MetalToolBarUI.components = new ArrayList();
    }
    
    protected class MetalContainerListener extends BasicToolBarUI.ToolBarContListener
    {
    }
    
    protected class MetalRolloverListener extends BasicToolBarUI.PropertyListener
    {
    }
    
    protected class MetalDockingListener extends BasicToolBarUI.DockingListener
    {
        private boolean pressedInBumps;
        
        public MetalDockingListener(JToolBar t) {
            super(t);
            this.pressedInBumps = false;
        }
        
        @Override
        public void mousePressed(MouseEvent e) {
            super.mousePressed(e);
            if (!this.toolBar.isEnabled()) {
                return;
            }
            this.pressedInBumps = false;
            Rectangle rectangle = new Rectangle();
            if (this.toolBar.getOrientation() == 0) {
                rectangle.setBounds(MetalUtils.isLeftToRight(this.toolBar) ? 0 : (this.toolBar.getSize().width - 14), 0, 14, this.toolBar.getSize().height);
            }
            else {
                rectangle.setBounds(0, 0, this.toolBar.getSize().width, 14);
            }
            if (rectangle.contains(e.getPoint())) {
                this.pressedInBumps = true;
                Point point = e.getPoint();
                if (!MetalUtils.isLeftToRight(this.toolBar)) {
                    point.x -= this.toolBar.getSize().width - this.toolBar.getPreferredSize().width;
                }
                MetalToolBarUI.this.setDragOffset(point);
            }
            return;
        }
        
        @Override
        public void mouseDragged(MouseEvent e) {
            if (this.pressedInBumps) {
                super.mouseDragged(e);
            }
        }
    }
}
